#pragma once

#include <windows.h>

typedef int (__stdcall * PluginProc)(LPVOID);		// prototype of plugin function
typedef int (__stdcall * MacroProc)(LPWSTR, DWORD);	// prototype of macro function
typedef int (__stdcall * APIProc)(UINT, LPVOID);	// prototype of API adapter function

typedef struct tagAutorunInterface
{
	DWORD	dwSize;		// struct size
	HWND	hWnd;		// TC window handle
	LPVOID	lpAdapter;	// pointer to adapter function
} AutorunInterface;

 // parameters for plugin functions
typedef struct tagAutorunFuncParams
{
	DWORD	dwSize;		// struct size
	LPCWSTR	Param;		// parameters string
	LPWSTR	Value;		// return value
	DWORD	dwBufSize;	// return value buffer size in BYTES
	INT		nError;		// user error value (non-fatal)
} AutorunFuncParams;

// structure for setting functions
typedef struct tagPluginFuncInfo
{
	DWORD	dwSize;		// struct size
	DWORD	dwFlags;	// function flags
	LPCWSTR	lpFuncName;	// name of function
	LPVOID	lpFuncAddr;	// function address
} PluginFuncInfo;

typedef struct tagVarInfo
{
	DWORD	dwSize;		// struct size
	DWORD	dwFlags;	// variable flags
	LPCWSTR	lpName;		// variable name
	LPVOID	lpData;		// variable value, buffer or function address
	DWORD	dwBufSize;	// buffer size in BYTES
} VarInfo;

// Error reporting
typedef struct tagErrorReport
{
	DWORD	dwSize;	// struct size
	INT	nMsgID;		// message ID
	LPCWSTR	lpText;	// message text / std param
	INT nFlags;		// reserved for future use
} ErrorReport;

// common struct for API functions that receive data
typedef struct tagGetData
{
	LPWSTR	lpText;		// text/name/caption
	LPWSTR	lpBuffer;	// buffer
	DWORD	dwBufSize;	// buffer size in BYTES
} ExpVars, ParseNextData, CallCmd;

// get value struct 
typedef struct tagValueInfo
{
	DWORD	dwSize;			// struct size
	DWORD	dwValueType;	// value type
	DWORD	dwFlags;		// optional flags
	LPCWSTR	lpName;			// value name if required
	LPVOID	lpData;			// value
	DWORD	dwBufSize;		// buffer size in BYTES
} ValueInfo;

// common result
#define AU_RESULT_OK	0
#define AU_RESULT_ERROR	1

// API function return values (error codes)
#define AU_RESULT_VARNOEXIST		2
#define AU_RESULT_NOREDEFINECONST	3

// Parser function return values
#define AU_PARSER_NOMOREPARAMS	0
#define AU_PARSER_REGULARPARAM	1
#define AU_PARSER_NAMEDPARARAM	2

// Flags for set variables proc
#define AU_VAR_CONSTANT	1
#define AU_VAR_MACRO	2

// Flags for set functions proc
#define AU_FUNC_RETURNVALUE	1
#define AU_FUNC_BOOLEAN		2

// Indexes of API functions
#define AU_PROC_ADDFUNC		1
#define AU_PROC_PARSER_NEXT	2
#define AU_PROC_GETVAR		3
#define AU_PROC_SETVAR		4
#define AU_PROC_EXPANDVARS	5
#define AU_PROC_REPORTERROR	6
#define AU_PROC_CALLCOMMAND 7
#define AU_PROC_GETVALUE    8

// Indexes for GetAutorunValue function
#define AU_VALUE_TCCOMMAND  1

#define AU_ERR_USERMSG		0
#define AU_ERR_UNSUPPORTED	4

void SetAdapter(LPVOID aAdapter);
int SetVar(LPWSTR AName, LPWSTR AValue, int AFlags);
int GetVar(LPWSTR AVarName, LPWSTR lpBuffer, UINT nBufferSize);
int SetVarMacro(LPWSTR AName, LPVOID AFuncAddr);
int AddFunc(LPWSTR AName, LPVOID AAddr, int AFlags);
int ParseNext(LPWSTR AParamStr, LPWSTR ABuffer, DWORD nBufSize);
int ExpandVars(LPWSTR AText, LPWSTR ABuffer, DWORD nBufSize);
int ReportError(INT MsgID, LPCWSTR aParam);
int ReportError(LPCWSTR aText);
int CallCommand(LPWSTR ACmdLine, LPWSTR AResult, DWORD nResultSize);
int CallCommand(LPWSTR ACmdLine);
int GetAutorunValue(DWORD AValueType, LPCWSTR AValueName, LPVOID ABuffer, DWORD nBufSize, DWORD AFlags = 0);
